/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkPointNormalColorReader.cxx,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#include "vtkPointNormalColorReader.h"

#include "vtkCellArray.h"
#include "vtkTable.h"
#include "vtkFloatArray.h"
#include "vtkInformation.h"
#include "vtkInformationVector.h"
#include "vtkObjectFactory.h"
#include "vtkPointData.h"
#include "vtkPolyData.h"
#include "vtkTriangle.h"
#include "vtkSmartPointer.h"
#include "vtkDelimitedTextReader.h"
#include "vtkDoubleArray.h"

#include <sstream>
#include <fstream>

vtkCxxRevisionMacro(vtkPointNormalColorReader, "$Revision: 1.27 $");
vtkStandardNewMacro(vtkPointNormalColorReader);

// Description:
// Instantiate object with NULL filename.
vtkPointNormalColorReader::vtkPointNormalColorReader()
{
  this->FileName = NULL;

  this->SetNumberOfInputPorts(0);
}

vtkPointNormalColorReader::~vtkPointNormalColorReader()
{
  if (this->FileName)
  {
    delete [] this->FileName;
    this->FileName = NULL;
  }
}


int vtkPointNormalColorReader::RequestData(
                              vtkInformation *vtkNotUsed(request),
    vtkInformationVector **vtkNotUsed(inputVector),
                                      vtkInformationVector *outputVector)
{
  // get the info object
  vtkInformation *outInfo = outputVector->GetInformationObject(0);

  // get the ouptut
  vtkPolyData *output = vtkPolyData::SafeDownCast(
      outInfo->Get(vtkDataObject::DATA_OBJECT()));

  if (!this->FileName) 
  {
    vtkErrorMacro(<< "A FileName must be specified.");
    return 0;
  }
    
  vtkSmartPointer<vtkDelimitedTextReader> Reader = vtkSmartPointer<vtkDelimitedTextReader>::New();
  Reader->SetFileName(this->FileName);
  Reader->DetectNumericColumnsOn();
  Reader->SetFieldDelimiterCharacters(" ");
  Reader->Update();
     
  //setup points/vertices
  vtkSmartPointer<vtkPoints> Points = vtkSmartPointer<vtkPoints>::New();
  vtkSmartPointer<vtkCellArray> Vertices = vtkSmartPointer<vtkCellArray>::New();
  
  //setup normals
  vtkSmartPointer<vtkDoubleArray> Normals = vtkSmartPointer<vtkDoubleArray>::New();
  Normals->SetNumberOfComponents(3); //3d normals (ie x,y,z)

  //setup colors
  vtkSmartPointer<vtkUnsignedCharArray> Colors = vtkSmartPointer<vtkUnsignedCharArray>::New();
  Colors->SetNumberOfComponents ( 3 );
  Colors->SetName ( "Colors" );
  
  //get the contents of the file
  vtkTable* Table = Reader->GetOutput();
  
  for(unsigned int i = 0; i < Table->GetNumberOfRows(); i++)
    {
    //vtkstd::cout << "x: " << (Table->GetValue(i,0)).ToDouble() << " y: " << (Table->GetValue(i,1)).ToDouble() << " z: " << (Table->GetValue(i,2)).ToDouble();
  
    
    //set points  
    vtkIdType pid[1];
    pid[0] = Points->InsertNextPoint((Table->GetValue(i,0)).ToDouble(), (Table->GetValue(i,1)).ToDouble(), (Table->GetValue(i,2)).ToDouble());
    Vertices->InsertNextCell ( 1,pid );
    
    //set normals
    double n[3];
    n[0] = (Table->GetValue(i,3)).ToDouble();
    n[1] = (Table->GetValue(i,4)).ToDouble();
    n[2] = (Table->GetValue(i,5)).ToDouble();
  
    //vtkstd::cout << " n: " << n[0] << " " << n[1] << " " << n[2] << vtkstd::endl;
    Normals->InsertNextTuple(n);

    //set colors
    unsigned char c[3];
    c[0] = (Table->GetValue(i,6)).ToUnsignedChar();
    c[1] = (Table->GetValue(i,7)).ToUnsignedChar();
    c[2] = (Table->GetValue(i,8)).ToUnsignedChar();

    Colors->InsertNextTupleValue ( c );  
    }

  
  output->SetPoints(Points);
  output->SetVerts(Vertices);
  output->GetPointData()->SetNormals(Normals);
  output->GetPointData()->AddArray (Colors);
  
  return 1;
}

void vtkPointNormalColorReader::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);

  os << indent << "File Name: " 
      << (this->FileName ? this->FileName : "(none)") << "\n";

}
